@@ -84,6 +84,12 @@ class Horde_ActiveSync_Device
      */
     const QUIRK_NEEDS_SUPPORTED_PICTURE_TAG = 1;
 
+    /**
+     * iOS sends an empty picture tag on every edit of contacts whose pictures
+     * did not originate on the client itself.
+     */
+    const QUIRK_INCORRECTLY_SENDS_EMPTY_PICTURE_TAG = 2;
+
     /**
      * Device properties.
      *
@@ -438,7 +444,7 @@ class Horde_ActiveSync_Device
         // First see if we have a newer client that sends the OS version
         // Newer iOS sends e.g., "iOS 8.2.2" in OS field.
         if (!empty($this->properties[self::OS]) &&
-            preg_match('/\d+\.\d+\.?\d+?/', $this->properties[self::OS], $matches)) {
+            preg_match('/\d+\.\d+\.?\d?/', $this->properties[self::OS], $matches)) {
             if (!empty($matches[0])) {
                 $this->_iOSVersion = $matches[0];
                 return;
@@ -553,7 +559,7 @@ class Horde_ActiveSync_Device
      *     Given all of this, it makes sense to me to ALWAYS send birthday
      *     data as occuring at 08:00:00 UTC for *native* Android clients.
      *
-     *   BB 10+ expects it at 12:00:00 UTC
+     *   BB 10+ expects it at 11:00:00 UTC
      *
      * @param Horde_Date $date  The date. This should normally be in the local
      *                          timezone if encoding the date for the client.
@@ -626,14 +632,14 @@ class Horde_ActiveSync_Device
 
         case self::TYPE_NINE:
                 if ($toEas) {
-                    return new Horde_Date($date->format('Y-m-d 00:00:00'));
+                    return new Horde_Date($date->format('Y-m-d 00:00:00'), 'UTC');
                 } else {
                     return $date;
                 }
 
         case self::TYPE_BLACKBERRY:
             if ($toEas) {
-                return new Horde_Date($date->format('Y-m-d 12:00:00'), 'UTC');
+                return new Horde_Date($date->format('Y-m-d 11:00:00'), 'UTC');
             } else {
                 $date = new Horde_Date($date->format('Y-m-d'));
                 return $date->setTimezone('UTC');
@@ -657,11 +663,11 @@ class Horde_ActiveSync_Device
     {
         switch ($quirk) {
             case self::QUIRK_NEEDS_SUPPORTED_PICTURE_TAG:
-                if ($this->_isIos() && $this->getMajorVersion() == 4) {
-                    return true;
-                }
-                return false;
-                break;
+                return ($this->_isIos() && $this->getMajorVersion() == 4);
+
+            case self::QUIRK_INCORRECTLY_SENDS_EMPTY_PICTURE_TAG:
+                return ($this->_isIos() && $this->getMajorVersion() >= 8 && $this->getMinorVersion() > 1);
+
             default:
                 return false;
         }
