@@ -38,13 +38,6 @@ class Horde_ActiveSync_Folder_RI extends Horde_ActiveSync_Folder_Base implements
     protected $_removed = array();
     protected $_added = array();
 
-    /**
-     * Flag for indicating we have an initial sync for this collection.
-     *
-     * @var boolean
-     */
-    public $haveInitialSync = false;
-
     /**
      * Set the current Recipient Cache
      *
