@@ -75,6 +75,8 @@ class Horde_ActiveSync_Imap_Adapter
      * @param string|stream $msg   The message
      * @param array $flags         Any flags to set on the newly appended message.
      *
+     * @return integer|boolean     The imap uid of the appended message or false
+     *                             on failure. @since 2.37.0
      * @throws new Horde_ActiveSync_Exception, Horde_ActiveSync_Exception_FolderGone
      */
     public function appendMessage($folderid, $msg, array $flags = array())
@@ -82,13 +84,18 @@ class Horde_ActiveSync_Imap_Adapter
         $message = array(array('data' => $msg, 'flags' => $flags));
         $mbox = new Horde_Imap_Client_Mailbox($folderid);
         try {
-            $this->_getImapOb()->append($mbox, $message);
+            $ids = $this->_getImapOb()->append($mbox, $message);
         } catch (Horde_Imap_Client_Exception $e) {
             if (!$this->_mailboxExists($folderid)) {
                 throw new Horde_ActiveSync_Exception_FolderGone();
             }
             throw new Horde_ActiveSync_Exception($e);
         }
+        if (!count($ids->ids)) {
+            return false;
+        }
+
+        return array_pop($ids->ids);
     }
 
     /**
@@ -114,7 +121,7 @@ class Horde_ActiveSync_Imap_Adapter
             $imap->subscribeMailbox($mbox, true);
         } catch (Horde_Imap_Client_Exception $e) {
             if ($e->getCode() == Horde_Imap_Client_Exception::ALREADYEXISTS) {
-                $this->_logger(sprintf(
+                $this->_logger->warn(sprintf(
                     '[%s] Mailbox %s already exists, subscribing to it.',
                     $this->_procid,
                     $name
@@ -335,28 +342,50 @@ class Horde_ActiveSync_Imap_Adapter
             // If we can't status the mailbox, assume it's gone.
             throw new Horde_ActiveSync_Exception_FolderGone($e);
         }
+
         $this->_logger->info(sprintf(
             '[%s] IMAP status: %s',
             $this->_procid,
             serialize($status))
         );
 
-        $current_modseq = $status[Horde_ActiveSync_Folder_Imap::HIGHESTMODSEQ];
-
-        if (($current_modseq && $folder->modseq() > 0) &&
-            (($folder->modseq() < $current_modseq) ||
-             !empty($options['softdelete']) || !empty($options['refreshfilter']) )) {
-
-            $this->_logger->info(sprintf(
-                '[%s] CONDSTORE and CHANGES', $this->_procid));
+        // UIDVALIDITY
+        if ($folder->uidnext() != 0) {
             $folder->checkValidity($status);
+        }
+
+        // If IMAP server reports invalid MODSEQ, this can lead to the client
+        // no longer ever able to detect changes therefore never receiving new
+        // email even if the value is restored at some point in the future.
+        //
+        // This can happen, e.g., if the IMAP server index files are lost or
+        // otherwise corrupted. Normally this would be handled as a loss of
+        // server state and handled by a complete resync, but a majority of
+        // EAS clients do not properly handle the status codes that report this.
+        $current_modseq = $status[Horde_ActiveSync_Folder_Imap::HIGHESTMODSEQ];
+        if ($modseq_corrupted = $folder->modseq() > $current_modseq) {
+            $this->_logger->err(sprintf(
+                '[%s] IMAP Server error: Current HIGHESTMODSEQ is lower than previously reported.',
+                 $this->_procid)
+            );
+        }
+
+        if ($modseq_corrupted || (($current_modseq && $folder->modseq() > 0) &&
+            (($folder->modseq() < $current_modseq) ||
+             !empty($options['softdelete']) || !empty($options['refreshfilter'])))) {
 
-            // Catch all *changes* since the provided MODSEQ value.
             $query = new Horde_Imap_Client_Search_Query();
-            // $imap->search uses a >= comparison for MODSEQ, so we must
-            // increment by one so we don't continuously receive the same change
-            // set.
-            $query->modseq($folder->modseq() + 1);
+
+            if (!$modseq_corrupted) {
+                $this->_logger->info(sprintf(
+                    '[%s] CONDSTORE and CHANGES', $this->_procid));
+                // Catch all *changes* since the provided MODSEQ value.
+                // $imap->search uses a >= comparison for MODSEQ, so we must
+                // increment by one so we don't continuously receive the same change
+                // set.
+                $query->modseq($folder->modseq() + 1);
+            }
+
             if (!empty($options['sincedate'])) {
                 $query->dateSearch(
                     new Horde_Date($options['sincedate']),
@@ -367,6 +396,7 @@ class Horde_ActiveSync_Imap_Adapter
                 $query,
                 array('results' => array(Horde_Imap_Client::SEARCH_RESULTS_MATCH))
             );
+
             $search_uids = $search_ret['count'] ?
                 $search_ret['match']->ids :
                 array();
@@ -396,7 +426,9 @@ class Horde_ActiveSync_Imap_Adapter
             // to no filter at all.
             $cnt = (count($search_uids) / self::MAX_FETCH) + 1;
             $query = new Horde_Imap_Client_Fetch_Query();
-            $query->modseq();
+            if (!$modseq_corrupted) {
+                $query->modseq();
+            }
             $query->flags();
             $changes = array();
             $categories = array();
@@ -501,7 +533,6 @@ class Horde_ActiveSync_Imap_Adapter
             $this->_logger->info(sprintf(
                 '[%s] NO CONDSTORE or per mailbox MODSEQ. minuid: %s, total_messages: %s',
                 $this->_procid, $folder->minuid(), $status['messages']));
-            $folder->checkValidity($status);
             $query = new Horde_Imap_Client_Search_Query();
             if (!empty($options['sincedate'])) {
                 $query->dateSearch(
@@ -545,7 +576,7 @@ class Horde_ActiveSync_Imap_Adapter
             }
             $folder->setRemoved($imap->vanished($mbox, null, array('ids' => new Horde_Imap_Client_Ids($folder->messages())))->ids);
         } elseif ($current_modseq > 0 && $folder->modseq() == 0) {
-                throw new Horde_ActiveSync_Exception_StaleState('Transition to MODSEQ enabled server');
+            throw new Horde_ActiveSync_Exception_StaleState('Transition to MODSEQ enabled server');
         }
         $folder->setStatus($status);
 
@@ -782,6 +813,12 @@ class Horde_ActiveSync_Imap_Adapter
             throw new Horde_ActiveSync_Exception($e);
         }
 
+        $this->_logger->info(sprintf(
+            '[%s] IMAP status: %s',
+            $this->_procid,
+            serialize($status))
+        );
+
         // If we have per mailbox MODSEQ then we can pick up flag changes too.
         $modseq = $status[Horde_ActiveSync_Folder_Imap::HIGHESTMODSEQ];
         if ($modseq && $folder->modseq() > 0 && $folder->modseq() < $modseq) {
@@ -1055,17 +1092,26 @@ class Horde_ActiveSync_Imap_Adapter
 
         // Fill in other header data
         try {
-            $eas_message->from = $imap_message->getFromAddress();
+            $eas_message->from = Horde_ActiveSync_Utils::ensureUtf8(
+                $imap_message->getFromAddress(),
+                'UTF-8'
+            );
         } catch (Horde_ActiveSync_Exception $e) {
             $this->_logger->err($e->getMessage());
         }
         try {
-            $eas_message->cc = $imap_message->getCc();
+            $eas_message->cc = Horde_ActiveSync_Utils::ensureUtf8(
+                $imap_message->getCc(),
+                'UTF-8'
+            );
         } catch (Horde_ActiveSync_Exception $e) {
             $this->_logger->err($e->getMessage());
         }
         try {
-            $eas_message->reply_to = $imap_message->getReplyTo();
+            $eas_message->reply_to = Horde_ActiveSync_Utils::ensureUtf8(
+                $imap_message->getReplyTo(),
+                'UTF-8'
+            );
         } catch (Horde_ActiveSync_Exception $e) {
             $this->_logger->err($e->getMessage());
         }
@@ -1074,7 +1120,7 @@ class Horde_ActiveSync_Imap_Adapter
         // angry. And we don't like angry clients.
         $eas_message->subject = Horde_ActiveSync_Utils::ensureUtf8(
             $imap_message->getSubject(),
-            $imap_message->getStructure()->getHeaderCharset()
+            'UTF-8'
         );
         $eas_message->threadtopic = $eas_message->subject;
         $eas_message->datereceived = $imap_message->getDate();
@@ -1310,7 +1356,10 @@ class Horde_ActiveSync_Imap_Adapter
         if ($version >= Horde_ActiveSync::VERSION_TWELVE) {
             $eas_message->contentclass = 'urn:content-classes:message';
             if ($mime_part = $imap_message->hasiCalendar()) {
-                $data = Horde_ActiveSync_Utils::ensureUtf8($mime_part->getContents(), $mime_part->getCharset());
+                $data = Horde_ActiveSync_Utils::ensureUtf8(
+                    $mime_part->getContents(),
+                    $mime_part->getCharset()
+                );
                 $vCal = new Horde_Icalendar();
                 if ($vCal->parsevCalendar($data, 'VCALENDAR', $mime_part->getCharset())) {
                     $classes = $vCal->getComponentClasses();
