@@ -36,7 +36,7 @@ class Horde_ActiveSync_Imap_Message
     /**
      * Message data.
      *
-     * @var Horde_Imap_Client_Fetch_Data
+     * @var Horde_Imap_Client_Data_Fetch
      */
     protected $_data;
 
@@ -329,11 +329,7 @@ class Horde_ActiveSync_Imap_Message
         }
         $atc->attsize = intval($mime_part->getBytes(true));
         $atc->attname = $this->_mbox . ':' . $this->uid . ':' . $id;
-        $atc->displayname = Horde_String::convertCharset(
-            $this->getPartName($mime_part, true),
-            $this->basePart->getHeaderCharset(),
-            'UTF-8',
-            true);
+        $atc->displayname = $this->getPartName($mime_part, true);
         $atc->attmethod = in_array($mime_part->getType(), array('message/rfc822', 'message/disposition-notification'))
             ? Horde_ActiveSync_Message_AirSyncBaseAttachment::ATT_TYPE_EMBEDDED
             : Horde_ActiveSync_Message_AirSyncBaseAttachment::ATT_TYPE_NORMAL;
@@ -594,15 +590,15 @@ class Horde_ActiveSync_Imap_Message
      * Return the To addresses from this message.
      *
      * @return array  An array containing arrays of 'to' and 'displayto'
-     *                addresses.
+     *                addresses. @since 2.37.1, ensures the text is UTF8.
      */
     public function getToAddresses()
     {
         $to = $this->envelope->to;
         $dtos = $tos = array();
         foreach ($to->raw_addresses as $e) {
-            $tos[] = $e->bare_address;
-            $dtos[] = $e->label;
+            $tos[] = Horde_ActiveSync_Utils::ensureUtf8($e->bare_address, 'UTF-8');
+            $dtos[] = Horde_ActiveSync_Utils::ensureUtf8($e->label, 'UTF-8');
         }
 
         return array('to' => $tos, 'displayto' => $dtos);
