@@ -336,12 +336,18 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
         $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
         $this->_encoder->content(self::STATUS_SUCCESS);
         $this->_encoder->endTag();
+
+        // Start SYNC_FOLDERS
         $this->_encoder->startTag(Horde_ActiveSync::SYNC_FOLDERS);
 
-        $exporter = new Horde_ActiveSync_Connector_Exporter(
+        // Get the exporter.
+        $exporter = new Horde_ActiveSync_Connector_Exporter_Sync(
             $this->_activeSync,
-            $this->_encoder);
+            $this->_encoder
+        );
 
+        // Loop through each collection and send all changes, replies, fetchids
+        // etc...
         $cnt_global = 0;
         $over_window = false;
         foreach ($this->_collections as $id => $collection) {
@@ -353,6 +359,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 continue;
             }
 
+            // Initialize this collection's state.
             try {
                 $this->_collections->initCollectionState($collection);
             } catch (Horde_ActiveSync_Exception_StateGone $e) {
@@ -388,6 +395,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     $this->_procid));
             }
 
+            // Check for server-side changes, if requested.
             if ($statusCode == self::STATUS_SUCCESS && !empty($collection['getchanges'])) {
                 try {
                     $changecount = $this->_collections->getCollectionChangeCount();
@@ -412,6 +420,13 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                         '[%s] FOLDERSYNC required, collection gone.',
                         $this->_procid));
                     $statusCode = self::STATUS_FOLDERSYNC_REQUIRED;
+                } catch (Horde_ActiveSync_Exception_TemporaryFailure $e) {
+                    $this->_logger->err(sprintf(
+                        '[%s] Failure in polling for changes: "%s".',
+                        $this->_procid,
+                        $e->getMessage())
+                    );
+                    $statusCode = Horde_ActiveSync_Status::SERVER_ERROR_RETRY;
                 } catch (Horde_Exception_AuthenticationFailure $e) {
                     $this->_logger->err(sprintf(
                         '[%s] Lost authentication during SYNC!!',
@@ -447,33 +462,22 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 }
             }
 
+            // Start SYNC_FOLDER
             $this->_encoder->startTag(Horde_ActiveSync::SYNC_FOLDER);
 
-            // Not sent in > 12.0
-            if ($this->_device->version <= Horde_ActiveSync::VERSION_TWELVE) {
-                $this->_encoder->startTag(Horde_ActiveSync::SYNC_FOLDERTYPE);
-                $this->_encoder->content($collection['class']);
-                $this->_encoder->endTag();
-            }
+            //SYNC_FOLDERTYPE
+            $exporter->syncFolderType($collection);
 
-            $this->_encoder->startTag(Horde_ActiveSync::SYNC_SYNCKEY);
-            if (!empty($collection['newsynckey'])) {
-                $this->_encoder->content($collection['newsynckey']);
-            } else {
-                $this->_encoder->content($collection['synckey']);
-            }
-            $this->_encoder->endTag();
+            // SYNC_KEY
+            $exporter->syncKey($collection);
 
-            $this->_encoder->startTag(Horde_ActiveSync::SYNC_FOLDERID);
-            $this->_encoder->content($collection['id']);
-            $this->_encoder->endTag();
+            // SYNC_FOLDERID
+            $exporter->syncFolderId($collection);
 
-            $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
-            $this->_encoder->content($statusCode);
-            $this->_encoder->endTag();
+            // SYNC_STATUS
+            $exporter->syncStatus($statusCode);
 
             if ($statusCode == self::STATUS_SUCCESS) {
-
                 // Server changes
                 if ($statusCode == self::STATUS_SUCCESS &&
                     empty($forceChanges) &&
@@ -483,6 +487,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                         ? min($collection['windowsize'], $pingSettings['maximumwindowsize'])
                         : $collection['windowsize'];
 
+                    // MOREAVAILABLE?
                     if (!empty($changecount) &&
                         (($changecount > $max_windowsize) || $cnt_global + $changecount > $this->_collections->getDefaultWindowSize())) {
                         $this->_logger->info(sprintf(
@@ -492,130 +497,65 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                         $over_window = ($cnt_global + $changecount > $this->_collections->getDefaultWindowSize());
                     }
 
+                    // Send each message now.
                     if (!empty($changecount)) {
                         $exporter->setChanges($this->_collections->getCollectionChanges(false), $collection);
+                        // Start SYNC_COMMANDS
                         $this->_encoder->startTag(Horde_ActiveSync::SYNC_COMMANDS);
                         $cnt_collection = 0;
                         while ($cnt_collection < $max_windowsize &&
                                $cnt_global < $this->_collections->getDefaultWindowSize() &&
                                $progress = $exporter->sendNextChange()) {
-
                             $this->_logger->info(sprintf(
                                 '[%s] Peak memory usage after message: %d',
-                                $this->_procid, memory_get_peak_usage(true)));
+                                $this->_procid, memory_get_peak_usage(true))
+                            );
                             if ($progress === true) {
                                 ++$cnt_collection;
                                 ++$cnt_global;
                             }
                         }
+                        // End SYNC_COMMANDS
                         $this->_encoder->endTag();
                     }
                 }
 
+                // Check for SYNC_REPLIES
                 if (!empty($collection['clientids']) || !empty($collection['fetchids'])
-                    || !empty($collection['missing']) || !empty($collection['importfailures'])) {
+                    || !empty($collection['missing']) || !empty($collection['importfailures'])
+                    || !empty($collection['modifiedids'])) {
 
+                    // Start SYNC_REPLIES
                     $this->_encoder->startTag(Horde_ActiveSync::SYNC_REPLIES);
 
                     // SYNC_MODIFY failures
                     if (!empty($collection['importfailures'])) {
-                        foreach ($collection['importfailures'] as $id => $reason) {
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_MODIFY);
+                        $exporter->modifyFailures($collection);
+                    }
 
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_FOLDERTYPE);
-                            $this->_encoder->content($collection['class']);
-                            $this->_encoder->endTag();
-
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_SERVERENTRYID);
-                            $this->_encoder->content($id);
-                            $this->_encoder->endTag();
-
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
-                            $this->_encoder->content($reason);
-                            $this->_encoder->endTag();
-
-                            $this->_encoder->endTag();
-                        }
+                    // EAS 16. CHANGED responses for items that need one. This
+                    // is basically the results of any AirSyncBaseAttachments
+                    // actions on Appointment or Draft Email items.
+                    if ($this->_device->version >= Horde_ActiveSync::VERSION_SIXTEEN &&
+                        !empty($collection['modifiedids'])) {
+                        $exporter->syncModifiedResponse($collection);
                     }
 
                     // Server IDs for new items we received from client
                     if (!empty($collection['clientids'])) {
-                        foreach ($collection['clientids'] as $clientid => $serverid) {
-                            if ($serverid) {
-                                $status = self::STATUS_SUCCESS;
-                            } else {
-                                $status = self::STATUS_INVALID;
-                            }
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_ADD);
-                            // If we have clientids and a CLASS_EMAIL, this is
-                            // a SMS response.
-                            if ($collection['class'] == Horde_ActiveSync::CLASS_EMAIL) {
-                                $this->_encoder->startTag(Horde_ActiveSync::SYNC_FOLDERTYPE);
-                                $this->_encoder->content(Horde_ActiveSync::CLASS_SMS);
-                                $this->_encoder->endTag();
-                            }
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_CLIENTENTRYID);
-                            $this->_encoder->content($clientid);
-                            $this->_encoder->endTag();
-                            if ($status == self::STATUS_SUCCESS) {
-                                $this->_encoder->startTag(Horde_ActiveSync::SYNC_SERVERENTRYID);
-                                $this->_encoder->content($serverid);
-                                $this->_encoder->endTag();
-                            }
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
-                            $this->_encoder->content($status);
-                            $this->_encoder->endTag();
-                            $this->_encoder->endTag();
-                        }
+                        $exporter->syncAddResponse($collection);
                     }
 
                     // Errors from missing messages in REMOVE requests.
                     if (!empty($collection['missing'])) {
-                        foreach ($collection['missing'] as $uid) {
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_REMOVE);
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_CLIENTENTRYID);
-                            $this->_encoder->content($uid);
-                            $this->_encoder->endTag();
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
-                            $this->_encoder->content(self::STATUS_NOTFOUND);
-                            $this->_encoder->endTag();
-                            $this->_encoder->endTag();
-                        }
+                        $exporter->missingRemove($collection);
                     }
 
                     if (!empty($collection['fetchids'])) {
-                        foreach ($collection['fetchids'] as $fetch_id) {
-                            try {
-                                $data = $this->_driver->fetch($collection['serverid'], $fetch_id, $collection);
-                                $this->_encoder->startTag(Horde_ActiveSync::SYNC_FETCH);
-                                $this->_encoder->startTag(Horde_ActiveSync::SYNC_SERVERENTRYID);
-                                $this->_encoder->content($fetch_id);
-                                $this->_encoder->endTag();
-                                $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
-                                $this->_encoder->content(self::STATUS_SUCCESS);
-                                $this->_encoder->endTag();
-                                $this->_encoder->startTag(Horde_ActiveSync::SYNC_DATA);
-                                $data->encodeStream($this->_encoder);
-                                $this->_encoder->endTag();
-                                $this->_encoder->endTag();
-                            } catch (Horde_Exception_NotFound $e) {
-                                $this->_logger->err(sprintf(
-                                    '[%s] Unable to fetch %s',
-                                    $this->_procid,
-                                    $fetch_id)
-                                );
-                                $this->_encoder->startTag(Horde_ActiveSync::SYNC_FETCH);
-                                $this->_encoder->startTag(Horde_ActiveSync::SYNC_SERVERENTRYID);
-                                $this->_encoder->content($fetch_id);
-                                $this->_encoder->endTag();
-                                $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
-                                $this->_encoder->content(self::STATUS_NOTFOUND);
-                                $this->_encoder->endTag();
-                                $this->_encoder->endTag();
-                            }
-                        }
+                        $exporter->fetchIds($this->_driver, $collection);
                     }
 
+                    // End SYNC_REPLIES
                     $this->_encoder->endTag();
                 }
 
@@ -642,13 +582,17 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 }
             }
 
+            // End SYNC_FOLDER
             $this->_encoder->endTag();
             $this->_logger->info(sprintf(
                 '[%s] Collection output peak memory usage: %d',
                 $this->_procid, memory_get_peak_usage(true)));
         }
 
+        // End SYNC_FOLDERS
         $this->_encoder->endTag();
+
+        // End SYNC_SYNCHRONIZE
         $this->_encoder->endTag();
 
         if ($this->_device->version >= Horde_ActiveSync::VERSION_TWELVEONE) {
@@ -1048,9 +992,9 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     case Horde_ActiveSync::CLASS_CALENDAR:
                         $appdata = Horde_ActiveSync::messageFactory('Appointment');
                         $appdata->decodeStream($this->_decoder);
+                        // EAS 16.0 sends instanceid/serverid for exceptions.
                         if (!empty($instanceid) &&
                             $commandType == Horde_ActiveSync::SYNC_MODIFY) {
-                            // EAS 16.0 sends instanceid/serverid for exceptions.
                             $appdata->instanceid = $instanceid;
                         }
                         break;
@@ -1082,24 +1026,43 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 switch ($commandType) {
                 case Horde_ActiveSync::SYNC_MODIFY:
                     if (isset($appdata)) {
-                        $id = $importer->importMessageChange(
+                        $ires = $importer->importMessageChange(
                             $serverid, $appdata, $this->_device, false,
                             $collection['class'], $collection['synckey']
                         );
-                        if ($id && !is_array($id)) {
+                        if (is_array($ires) && !empty($ires['error'])) {
+                            $collection['importedfailures'][$ires[0]] = $ires['error'];
+                        } elseif (is_array($ires)) {
                             $collection['importedchanges'] = true;
-                        } elseif (is_array($id)) {
-                            $collection['importfailures'][$id[0]] = $id[1];
+                            if (empty($collection['modifiedids'])) {
+                                $collection['modifiedids'] = array();
+                            }
+                            $collection['modifiedids'][] = $ires['id'];
+                            $collection['atchash'][$serverid] = !empty($ires['atchash'])
+                                ? $ires['atchash']
+                                : array();
                         }
                     }
                     break;
 
                 case Horde_ActiveSync::SYNC_ADD:
                     if (isset($appdata)) {
-                        $id = $importer->importMessageChange(
-                            false, $appdata, $this->_device, $clientid, $collection['class']);
-                        if ($clientid && $id && !is_array($id)) {
-                            $collection['clientids'][$clientid] = $id;
+                        $ires = $importer->importMessageChange(
+                            false, $appdata, $this->_device, $clientid,
+                            $collection['class']
+                        );
+                        if (!$ires || !empty($ires['error'])) {
+                            $collection['clientids'][$clientid] = false;
+                        } elseif ($clientid && is_array($ires)) {
+                            $collection['clientids'][$clientid] = $ires['id'];
+                            $collection['atchash'][$ires['id']] = !empty($ires['atchash'])
+                                ? $ires['atchash']
+                                : array();
+                            if (!empty($ires['conversationid'])) {
+                                $collection['conversations'][$ires['id']] =
+                                    array($ires['conversationid'],
+                                          $ires['conversationindex']);
+                            }
                             $collection['importedchanges'] = true;
                         } elseif (!$id || is_array($id)) {
                             $collection['clientids'][$clientid] = false;
