@@ -171,7 +171,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
             . 'sync_devid = ? AND sync_user = ? AND sync_folderid = ?';
 
         try {
-            $results = $this->_db->selectAll($sql,
+            $results = $this->_db->select($sql,
                 array($this->_deviceInfo->id, $this->_deviceInfo->user, $uid));
         } catch (Horde_Db_Exception $e) {
             $this->_logger->err(sprintf(
@@ -478,6 +478,8 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
 
             // This is an incoming change from the client, store it so we
             // don't mirror it back to device.
+            // @todo: Use bitmask for the change type so we don't have to
+            // maintain a separate field for each type.
             switch ($this->_collection['class']) {
             case Horde_ActiveSync::CLASS_EMAIL:
                 if ($type == Horde_ActiveSync::CHANGE_TYPE_CHANGE &&
@@ -525,6 +527,14 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                         . ' VALUES (?, ?, ?, ?, ?, ?)';
                     $flag_value = true;
                     break;
+                case Horde_ActiveSync::CHANGE_TYPE_DRAFT:
+                    // Incoming draft messge.
+                    $sql = 'INSERT INTO ' . $this->_syncMailMapTable
+                        . ' (message_uid, sync_key, sync_devid,'
+                        . ' sync_folderid, sync_user, sync_draft)'
+                        . ' VALUES (?, ?, ?, ?, ?, ?)';
+                    $flag_value = true;
+                    break;
                 }
                 $params = array($change['id'], $syncKey, $this->_deviceInfo->id,
                     $change['serverid'], $user, $flag_value
@@ -1339,13 +1349,15 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      * @return array  An array of hashes, each in the form of
      *   {uid} => array(
      *     Horde_ActiveSync::CHANGE_TYPE_FLAGS => true|false,
-     *     Horde_ActiveSync::CHANGE_TYPE_DELETE => true|false
+     *     Horde_ActiveSync::CHANGE_TYPE_DELETE => true|false,
+     *     Horde_ActiveSync::CHANGE_TYPE_DRAFT  => true|false,
      *   )
      */
     protected function _getMailMapChanges(array $changes)
     {
         $sql = 'SELECT message_uid, sync_read, sync_flagged, sync_deleted,'
-            . 'sync_changed, sync_category FROM ' . $this->_syncMailMapTable
+            . 'sync_changed, sync_category, sync_draft FROM '
+            . $this->_syncMailMapTable
             . ' WHERE sync_folderid = ? AND sync_devid = ?'
             . ' AND sync_user = ? AND message_uid IN '
             . '(' . implode(',', array_fill(0, count($changes), '?')) . ')';
@@ -1362,7 +1374,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
             $ids);
 
         try {
-            $rows = $this->_db->selectAll($sql, $values);
+            $rows = $this->_db->select($sql, $values);
         } catch (Horde_Db_Exception $e) {
             throw new Horde_ActiveSync_Exception($e);
         }
@@ -1385,6 +1397,10 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                         $results[$row['message_uid']][$change['type']] =
                             !is_null($row['sync_changed']) && $row['sync_changed'] == true;
                         continue 3;
+                    case Horde_ActiveSync::CHANGE_TYPE_DRAFT:
+                        $results[$row['message_uid']][$change['type']] =
+                            !is_null($row['sync_draft']) && $row['sync_draft'] == true;
+                        continue 3;
                     }
                 }
             }
@@ -1447,7 +1463,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
             $this->_deviceInfo->user);
 
         try {
-            $results = $this->_db->selectAll($sql, $values);
+            $results = $this->_db->select($sql, $values);
         } catch (Horde_Db_Exception $e) {
             $this->_logger->err(sprintf(
                 '[%s] %s',
@@ -1573,4 +1589,21 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         $cache->save();
     }
 
+     /**
+      * Close the underlying backend storage connection.
+      * To be used during PING or looping SYNC operations.
+      */
+     public function disconnect()
+     {
+        $this->_db->disconnect();
+     }
+
+     /**
+      * (Re)open backend storage connection.
+      */
+     public function connect()
+     {
+        $this->_db->connect();
+     }
+
 }
