@@ -441,7 +441,7 @@ class Horde_ActiveSync_StateTest_Base extends Horde_Test_Case
 
     protected function _testPartialSyncWithOnlyChangedHbInterval()
     {
-        $this->markTestSkipped('No idea why the cache does not load the collections here.');
+        $this->markTestIncomplete('No idea why the cache does not load the collections here.');
         $collections = $this->getCollectionHandler();
         $collections->loadCollectionsFromCache();
         $collections->setHeartbeat(array('hbinterval' => 1));
@@ -499,8 +499,6 @@ class Horde_ActiveSync_StateTest_Base extends Horde_Test_Case
         );
         $collections->addCollection($col);
         $collections->initPartialSync();
-        $this->assertEquals(1, $collections->collectionCount());
-        $collections->getMissingCollectionsFromCache();
         $this->assertEquals(2, $collections->collectionCount());
     }
 
